@@ -32,8 +32,8 @@ class AsymptoticCLs(AnalysisObject):
                  ws_name:Optional[str]=None, mc_name:Optional[str]=None,
                  snapshot_name:Optional[str]=None, timer:bool=False,
                  minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', 
-                 eps:float=1.0, strategy:int=0, print_level:int=-1, 
-                 num_cpu:int=1, offset:bool=True, optimize:bool=True,
+                 eps:float=1.0, strategy:int=1, print_level:int=-1, 
+                 num_cpu:int=1, offset:bool=True, optimize:int=2,
                  fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1,
                  max_iters:int=-1, constrain_nuis:bool=True, batch_mode:bool=False,
                  int_bin_precision:float=-1., **kwargs):
@@ -188,7 +188,7 @@ class AsymptoticCLs(AnalysisObject):
         self.poi.setConstant(is_const)
         qmu = 2*(nll_value - nll_muhat)
         if qmu < 0 :
-            raise RuntimeError("qmu is negative which means the globally minimized"
+            raise RuntimeError("qmu is negative which means the globally minimized "
                                "NLL is larger then the NLL of interest")
         return qmu
 
@@ -263,7 +263,7 @@ class AsymptoticCLs(AnalysisObject):
         mu_pre = mu_hat
         mu_pre2 = mu_hat
         
-        while (abs(mu_pre-mu_guess) > self.precision*mu_guess*self.direction):
+        while (abs(mu_pre-mu_guess) > self.precision*mu_guess):
             print('----------------------------------')
             print('Starting iteration {} of {}'.format(n_iter, nll_name))
             # do this to avoid comparing multiple minima in the conditional and unconditional fits           
@@ -448,8 +448,7 @@ class AsymptoticCLs(AnalysisObject):
         print('Found N * sigma = {} * {}'.format(n, sigma))
         snapshot_0_name = self.nll_maps[asimov_0_nll]['snapshot']
         self.load_snapshot(snapshot_0_name)
-        asimov_data_n = self.model.generate_asimov(self.poi.GetName(), poi_val=n_times_sigma,
-                                                   do_conditional=True)
+        asimov_data_n = self.model.generate_asimov(self.poi.GetName(), poi_val=n_times_sigma)
         asimov_n_nll = self.create_nll(asimov_data_n)
         self.load_conditional_snapshot(n_times_sigma, target="globs")
         self.load_conditional_snapshot(n_times_sigma, target="nuis")
@@ -478,10 +477,12 @@ class AsymptoticCLs(AnalysisObject):
         
         self.save_snapshot(self.nom_glob_name, target="globs")
         self.save_snapshot(self.nom_nuis_name, target="nuis")
+        
+        # profile NP to mu = 0 if unblinded else no profiling
+        poi_profile = None if self.do_blind else 0
 
         asimov_data_0 = self.model.generate_asimov(self.poi.GetName(), poi_val=0,
-                                                   do_conditional=not self.do_blind,
-                                                   poi_profile=0, dataset=self.data)
+                                                   poi_profile=poi_profile, dataset=self.data)
         asimov_0_nll = self.create_nll(asimov_data_0)
         self.summary[0] = {'dataset': asimov_data_0, 'nll': asimov_0_nll}
         self.set_poi_value(0)
