@@ -20,7 +20,7 @@ class ExtendedMinimizer(object):
         'opt_const': 2,
         'verbose': 0,
         'save': 0,
-        'timer': 1,
+        'timer': 0,
         'hesse': 0,
         'minos': 0,
         'scan': 0,
@@ -35,8 +35,8 @@ class ExtendedMinimizer(object):
         'max_iters': -1,
         'n_sigma': 1,
         'precision': 1,
-        'default_strategy': 0,
-        'print_level': 1   
+        'default_strategy': 1,
+        'print_level': -1   
     }
     
     _DEFAULT_NLL_OPTION_ = {
@@ -305,6 +305,7 @@ class ExtendedMinimizer(object):
         retry = self.config['retry']
         minimizer_type = self.config['minimizer_type']
         minimizer_algo = self.config['minimizer_algo']
+
         status = self.minimizer.minimize(minimizer_type, minimizer_algo)
         
         while ((status != 0) and (status!=1) and (strategy<2) and (retry>0)):
