@@ -10,10 +10,12 @@ import logging
 import fnmatch
 from typing import List, Optional, Union, Dict
 
+import numpy as np
+
 import ROOT
 
 import quickstats
-from quickstats.components.numerics import is_integer
+from quickstats.components.numerics import is_integer, pretty_value
 from quickstats.utils.root_utils import load_macro
 from quickstats.utils.io import VerbosePrint
 
@@ -31,13 +33,13 @@ class ExtendedModel(object):
         'weight': 'weightVar',
         'dataset_args': 'obsAndWeight',
         'asimov': 'asimovData_{mu}',
-        'channel_asimov': 'combAsimovData{index}',
+        'channel_asimov': 'combAsimovData_{label}',
         'nll_snapshot': '{nll_name}_{mu}'
     }
     
     def __init__(self, fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None,
                  data_name:str="combData", snapshot_name:Optional[str]=None,
-                 binned_likelihood:bool=True, tag_as_measurement:str="pdf_",
+                 binned_likelihood:bool=True, tag_as_measurement:Optional[str]=None,
                  fix_cache:bool=True, fix_multi:bool=True, interpolation_code:int=-1,
                  load_extension:bool=True, verbosity:Union[int, str]="INFO"):
         self.fname = fname
@@ -115,7 +117,11 @@ class ExtendedModel(object):
     def activate_binned_likelihood(ws):
         _PRINT_.info('INFO: Activating binned likelihood evaluation')
         for component in ws.components():
-            if (component.IsA() == ROOT.RooRealSumPdf.Class()):
+            try:
+                flag = (component.IsA() == ROOT.RooRealSumPdf.Class())
+            except:
+                flag = (component.ClassName() == "RooRealSumPdf")
+            if (flag):
                 component.setAttribute('BinnedLikelihood')
                 _PRINT_.info('INFO: Activated binned likelihood attribute for {}'.format(component.GetName()))
         return None
@@ -125,7 +131,11 @@ class ExtendedModel(object):
         _PRINT_.info('INFO: Activating measurements to reduce memory consumption')
         for component in ws.components():
             name = component.GetName()
-            if ((component.IsA() == ROOT.RooAddPdf.Class()) and condition(name)):
+            try:
+                flag = (component.IsA() == ROOT.RooAddPdf.Class())
+            except:
+                flag = (component.ClassName() == "RooAddPdf")
+            if flag and condition(name):
                 component.setAttribute('MAIN_MEASUREMENT')
                 _PRINT_.info('INFO: Activated main measurement attribute for {}'.format(name))
         return None
@@ -686,7 +696,7 @@ class ExtendedModel(object):
         asimov_data = ROOT.RooDataSet(name, name, arg_set, ROOT.RooFit.WeightVar(weight_var))
         
         # generate observables defined by the pdf associated with this state
-        obs = pdf.getObservables(self.observables)        
+        obs = pdf.getObservables(self.observables)
         target_obs = obs.first()
         expected_events = pdf.expectedEvents(obs)
         #print("INFO: Generating Asimov for pdf {}".format(pdf.GetName()))
@@ -698,16 +708,71 @@ class ExtendedModel(object):
                 _PRINT_.warning("WARNING: Detected bin with zero expected events ({})! Please check"
                       "your inputs. Obs = {}, bin = {}".format(n_events, target_obs.GetName(), i))
             elif (n_events > 0) and (n_events < 1e18):
+                _PRINT_.debug("pdf={}, obs={}, bin={}, val={}".format(
+                    pdf.GetName(), target_obs.GetName(), i, n_events))
                 asimov_data.add(self.observables, n_events)
+            else:
+                raise RuntimeError(f"detected pdf bin with nan (pdf={pdf.GetName()},obs={target_obs.GetName()},bin={i})")
 
         if (asimov_data.sumEntries() != asimov_data.sumEntries()):
             raise RuntimeError("asimov data sum entries is nan")
         return asimov_data
 
     def generate_asimov(self, poi_name:str, poi_val:float, poi_profile:Optional[float]=None,
-                        do_conditional:bool=False, do_import:bool=True, dataset:Optional["ROOT.RooDataSet"]=None,
+                        conditional_mle:bool=True, do_import:bool=True,
+                        asimov_name:Optional[str]=None,
+                        channel_asimov_name:Optional[str]=None,
+                        dataset:Optional["ROOT.RooDataSet"]=None,
                         minimizer_options:Optional[Dict]=None, nll_options:Optional[Dict]=None,
-                        object_names:Optional[Dict]=None):
+                        snapshot_names:Optional[Dict]=None):
+        """
+            Generate Asimov dataset.
+            
+            Note:
+                Nominal (initial values) snapshots of nuisance parameters and global are saved
+                as "nominalNuis" and "nominalGlobs" if not already exist. Conditional snapshots
+                are saved as "conditionalNuis_{mu}" and "conditionalGlobs_{mu}" irrespective of
+                whether nuisance parameter profiling is performed and whether conditional mle
+                is used for the profiling. This is to faciliate the use case of Asymptotic limit
+                calculation. The names of these snahpshots can be customized via the
+                `snapshot_names` option.
+        
+            Arguments:
+                poi_name: str
+                    Name of the parameter of interest (POI).
+                poi_val: float
+                    Generate asimov data with POI set at the specified value.
+                poi_profile: (Optional) float
+                    Perform nuisance parameter profiling. If None, nuisance parameters are kept at their
+                    initial values when generating the asimov data. Otherwise, a fit is performed by setting 
+                    the POI at the specified value (POI is fixed if conditional_mle=True, free otherwise) to
+                    obtain the best-fit nuisance parameter values before generating the asimov data.
+                    category label. Other keywords are: `{index}` which is the category index.
+                do_import: bool, default=True
+                    Import the generated asimov data to the current workspace.                    
+                conditional_mle: bool, default=True
+                    Perform conditional maximum likelihood estimation. If True, best-fit values are obtained
+                    for mu = poi_profile. If False, the overal best-fit values are obtained.
+                asimov_name: (Optional) str
+                    Name of the generated asimov dataset. If None, defaults to "asimovData_{mu}" where
+                    `{mu}` will be replaced by the value of `poi_val`. Other keywords are: `{mu_cond}`
+                    which is the the value of `poi_profile`.
+                channel_asimov_name: (Optional) str
+                    Name of the asimov dataset in each category of a simultaneous pdf. If None,
+                    defaults to "combAsimovData_{label}" where `{label}` will be replaced by the
+                dataset: (Optional) ROOT.RooDataSet
+                    Dataset based on which the negative log likelihood (NLL) is created for nuisance parameter
+                    profiling. If None, default to self.data.
+                minimizer_options: (Optional) dict
+                    Options for minimization during nuisance parameter profiling. If None, defaults to
+                    ExtendedMinimizer._DEFAULT_MINIMIZER_OPTION_
+                nll_options: (Optional) dict
+                    Options for NLl creation during nuisance parameter profiling. If None, defaults to
+                    ExtendedMinimizer._DEFAULT_NLL_OPTION_                    
+                snapshot_names: (Optional) dict
+                    A dictionary containing a map of the snapshot type and the snapshot names. The default
+                    namings are stored in ExtendedModel._DEFAULT_NAMES_.
+        """
         # define simplified ws variable names
         ws = self.workspace
         globs = self.global_observables
@@ -715,15 +780,20 @@ class ExtendedModel(object):
         
         # define names used for various objects
         names = self._DEFAULT_NAMES_
-        if object_names is not None:
-            names.update(object_names)
+        if snapshot_names is not None:
+            names.update(snapshot_names)
+        if asimov_name is not None:
+            names['asimov'] = asimov_name
+        if channel_asimov_name is not None:
+            names['channel_asimov'] = channel_asimov_name
         tmp_glob_name = names['temp_globs']
         tmp_nuis_name = names['temp_nuis'] 
         nom_glob_name = names['nominal_globs']
         nom_nuis_name = names['nominal_nuis']
         con_glob_name = names['conditional_globs']
         con_nuis_name = names['conditional_nuis']
-        asimov_data_name = names['asimov'].format(mu=poi_val)
+        asimov_data_name = names['asimov'].format(mu=pretty_value(poi_val), 
+                                                  mu_cond=pretty_value(poi_profile))
         channel_asimov_data_name = names['channel_asimov']
         
         poi = self.get_poi(poi_name)
@@ -740,7 +810,7 @@ class ExtendedModel(object):
             _PRINT_.info('INFO: Saving snapshot "{}"'.format(nom_nuis_name))
             ws.saveSnapshot(nom_nuis_name, nuis)
 
-        if do_conditional and (poi_profile is not None):
+        if (poi_profile is not None):
             from quickstats.components import ExtendedMinimizer
             if dataset is None:
                 dataset = self.data
@@ -753,7 +823,8 @@ class ExtendedModel(object):
             minimizer.configure_nll(self.nuisance_parameters, self.global_observables,
                                     **nll_options)
             poi.setVal(poi_profile)
-            poi.setConstant(1)
+            if conditional_mle:
+                poi.setConstant(1)
             minimizer.minimize()
         poi.setVal(poi_val)
         poi.setConstant(0)
@@ -761,16 +832,15 @@ class ExtendedModel(object):
         for np, glob in zip(nuis_list, glob_list):
             glob.setVal(np.getVal())
 
-        if do_conditional and (poi_profile is not None):
+        if (poi_profile is not None):
             ws.saveSnapshot(con_glob_name.format(mu=poi_profile), globs)
             ws.saveSnapshot(con_nuis_name.format(mu=poi_profile), nuis)
         else:
             ws.saveSnapshot(con_glob_name.format(mu=poi_val), globs)
             ws.saveSnapshot(con_nuis_name.format(mu=poi_val), nuis)
-            
-        if not do_conditional:
             ws.loadSnapshot(nom_glob_name)
-            ws.loadSnapshot(nom_nuis_name)    
+            ws.loadSnapshot(nom_nuis_name)
+            
         poi.setVal(poi_val)
         
         sim_pdf = self.pdf
@@ -784,7 +854,7 @@ class ExtendedModel(object):
                 channel_cat.setIndex(i)
                 label = channel_cat.getLabel()
                 pdf_cat = sim_pdf.getPdf(label)
-                name = channel_asimov_data_name.format(index=i)
+                name = channel_asimov_data_name.format(index=i, label=label)
                 asimov_data_map[label] = self.generate_asimov_from_pdf(name, pdf_cat, extra_args=channel_cat)
 
             obs_and_weight, weight_var = self._load_obs_and_weight()
@@ -796,9 +866,14 @@ class ExtendedModel(object):
                                           ROOT.RooFit.WeightVar(weight_var))
         if do_import:
             getattr(ws, "import")(asimov_data)
+            
         ws.loadSnapshot(nom_glob_name)
+        
         return asimov_data
     
+    def save(self, fname:str, recreate:bool=True):
+        self.workspace.writeToFile(fname, recreate)
+    
     @staticmethod
     def to_dataframe(args):
         import pandas as pd
@@ -865,4 +940,188 @@ class ExtendedModel(object):
         datasets = ws.allData()
         if not datasets:
             raise RuntimeError('Failed to load datasets')
-        return [dataset.GetName() for dataset in datasets]
\ No newline at end of file
+        return [dataset.GetName() for dataset in datasets]
+    
+    def print_summary(self, show_workspace=True, show_pois=True, show_datasets=True,
+                      show_observables=True, show_nuisance_parameters=False):
+        if show_workspace:
+            print("workspace:", self.workspace.GetName(), "\n")
+        if show_pois:
+            print("POIs:", ", ".join([i.GetName() for i in self.pois]), "\n")
+        if show_datasets:
+            print("datasets:", ", ".join([i.GetName() for i in self.workspace.allData()]), "\n")
+        if show_observables:
+            print("observables:", ", ".join([i.GetName() for i in self.observables]), "\n")
+        if show_nuisance_parameters:
+            print("nuisance parameters:", ", ".join([i.GetName() for i in self.nuisance_parameters]), "\n")
+    
+    def get_category_map(self, pdf=None):
+        if pdf is None:
+            pdf = self.pdf
+        if not isinstance(pdf, ROOT.RooSimultaneous):
+            raise ValueError("input pdf is not a simultaneous pdf")            
+        category_map = {}
+        index_cat = pdf.indexCat()
+        n_cat = len(index_cat)
+        for i in range(n_cat):
+            index_cat.setIndex(i)
+            label = index_cat.getLabel()
+            pdf_cat = pdf.getPdf(label)
+            obs = pdf_cat.getObservables(self.observables)
+            target_obs = obs.first()
+            category_map[label] = {}
+            category_map[label]['index'] = i
+            category_map[label]['pdf'] = pdf_cat.GetName()
+            category_map[label]['observable'] = target_obs.GetName()
+        return category_map
+    
+    @staticmethod
+    def get_dataset_values(dataset):
+        parameters = dataset.get()
+        observables = []
+        category = None
+        for p in parameters:
+            if isinstance(p, ROOT.RooRealVar):
+                observables.append(p)
+            elif isinstance(p, ROOT.RooCategory):
+                if category is None:
+                    category = p
+                else:
+                    raise RuntimeError("multiple RooCategory instances found in the dataset")
+            else:
+                raise RuntimeError(f"unknown object type `{type(p)}` found in the dataset")
+        n_entries = dataset.numEntries()
+        result = {}
+        """
+        # only available after ROOT 6.24
+        data_buffer = ROOT.RooBatchCompute.RunContext()
+        dataset.getBatches(data_buffer, 0, n_entries)
+        for obs in observables:
+            obs_name = obs.GetName()
+            data = np.asarray(data_buffer.getBatch(obs))
+            result[obs_name] = data
+        """
+        for obs in observables:
+            obs_name = obs.GetName()
+            result[obs_name] = np.empty(n_entries)
+            for i in range(n_entries):
+                dataset.get(i)
+                result[obs_name][i] = obs.getVal()
+        result['weight'] = np.asarray(dataset.getWeightBatch(0, n_entries))
+        if category is not None:
+            result['label'] = np.empty(n_entries, dtype=object)
+            result['index'] = np.empty(n_entries)
+            for i in range(n_entries):
+                dataset.get(i)
+                result['label'][i] = category.getLabel()
+                result['index'][i] = category.getIndex()
+        return result
+    
+    def get_dataset_distributions(self, dataset):
+        distributions = {}
+        dataset_values = self.get_dataset_values(dataset)
+        if 'label' not in dataset_values:
+            raise RuntimeError("no categories defined in the dataset")
+        category_map = self.get_category_map()
+        for cat in category_map:
+            distributions[cat] = {}
+            observable = category_map[cat]['observable']
+            if observable not in dataset_values:
+                raise RuntimeError(f"no data associated with the observable `{observable}` found in the dataset")
+            distributions[cat]['observable'] = observable
+            mask = (dataset_values['label'] == cat)
+            x = dataset_values[observable][mask]
+            ind = np.argsort(x)
+            x = x[ind]
+            y = dataset_values['weight'][mask][ind]
+            distributions[cat]['x'] = x
+            distributions[cat]['y'] = y
+        return distributions
+    
+    @staticmethod
+    def get_simul_pdf_distributions(pdf, observables):
+        if not isinstance(pdf, ROOT.RooSimultaneous):
+            raise ValueError("input pdf is not a simultaneous pdf")
+        distributions = {}
+        channel_cat = pdf.indexCat()
+        n_cat = len(channel_cat)
+        for i in range(n_cat):
+            channel_cat.setIndex(i)
+            label = channel_cat.getLabel()
+            pdf_cat = pdf.getPdf(label)
+            obs = pdf_cat.getObservables(observables)
+            target_obs = obs.first()
+            expected_events = pdf_cat.expectedEvents(obs)
+            n_bins = target_obs.numBins()
+            obs_name = target_obs.GetName()
+            distributions[label] = {'x':[], 'y':[], 'observable': obs_name}
+            for i in range(n_bins):
+                target_obs.setBin(i)
+                norm = pdf_cat.getVal(obs)*target_obs.getBinWidth(i)
+                n_events = norm*expected_events
+                bin_value = target_obs.getVal()
+                if n_events <= 0:
+                    _PRINT_.warning("WARNING: Detected bin with zero expected events ({})! "
+                                    "Please check your inputs. Obs = {}, bin = {}".format(
+                                    n_events, target_obs.GetName(), i))
+                elif (n_events > 0) and (n_events < 1e18):
+                    distributions[label]['x'].append(bin_value)
+                    distributions[label]['y'].append(n_events)
+                else:
+                    raise RuntimeError(f"detected pdf bin with nan (pdf={pdf.GetName()},obs={target_obs.GetName()},bin={i})")
+        return distributions
+    
+    def get_distributions(self):
+        return self.get_simul_pdf_distributions(self.pdf, self.observables)
+    
+    def get_categories(self):
+        return list(self.get_category_map())
+    
+    def get_collected_distributions(self, current_distributions=True,
+                                    datasets:Optional[Union[List[str],List[ROOT.RooDataSet]]]=None,
+                                    snapshots:Optional[List[str]]=None):
+        collected_distributions = {}
+        
+        if current_distributions:
+            distributions = self.get_distributions()
+            for category in distributions:
+                collected_distributions[category] = {"Current": distributions[category]}
+                
+        if datasets is not None:
+            for dataset in datasets:
+                if isinstance(dataset, str):
+                    dataset = self.workspace.data(dataset)
+                dataset_name = dataset.GetName()
+                distributions = self.get_dataset_distributions(dataset)
+                for category in distributions:
+                    if category not in collected_distributions:
+                        collected_distributions[category] = {}
+                    collected_distributions[category][dataset_name] = distributions[category]
+        if snapshots is not None:
+            self.workspace.saveSnapshot("tmp", self.workspace.allVars())
+            try:
+                for snapshot in snapshots:
+                    exist = self.workspace.loadSnapshot(snapshot)
+                    if not exist:
+                        raise RuntimeError("snapshot `{}` not found in workspace".format(snapshot))
+                    distributions = self.get_distributions()
+                    for category in distributions:
+                        if category not in collected_distributions:
+                            collected_distributions[category] = {}
+                        collected_distributions[category][snapshot] = distributions[category]
+            finally:
+                self.workspace.loadSnapshot("tmp")
+        return collected_distributions
+            
+    def plot_distributions(self, category:str, current_distributions=True, 
+                           datasets:Optional[Union[List[str],List[ROOT.RooDataSet]]]=None,
+                           snapshots:Optional[Union[List[str], List[ROOT.RooArgSet]]]=None):
+        from quickstats.plots.template import plot_distributions
+        category_map = self.get_category_map()
+        if category not in category_map:
+            raise ValueError("category `{}` not found in workspace".format(category))
+        collected_distributions = self.get_collected_distributions(current_distributions,
+                                                                   datasets, snapshots=snapshots)
+        xlabel = category_map[category]['observable']
+        ylabel = "Events"
+        return plot_distributions(collected_distributions[category], xlabel=xlabel, ylabel=ylabel)
\ No newline at end of file
