@@ -24,7 +24,7 @@ def evaluate_nll(input_file:str, poi_val:float, poi:str="", vmin:float=10., vmax
                  minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', num_cpu:int=1, binned:bool=True,
                  eps:float=1.0, strategy:int=0, fix_cache:bool=True, fix_multi:bool=True, mpsplit:int=3, 
                  verbose:int=0, max_calls:int=-1, max_iters:int=-1, optimize:int=2, offset:bool=False, 
-                 outname:str=None, cache:bool=False):
+                 outname:str=None, cache:bool=False, detailed_output:bool=False):
     if cache:
         cached_result = load_cache(outname)
         if cached_result is not None:
@@ -90,21 +90,28 @@ def evaluate_nll(input_file:str, poi_val:float, poi:str="", vmin:float=10., vmax
         print('INFO: NLL for POI "{}" at {:.2f}: {}'.format(poi.GetName(), poi_val, nll))
     print('INFO: Time taken to evaluate NLL: {:.3f} s'.format(end-start))
     
+    results = {
+        'nll': nll,
+        'poi': poi.GetName(),
+        'constrain': int(constrain),
+        'poi_value': poi_val,
+        'poi_min': poi_min,
+        'poi_max': poi_max,
+        'unconditional': int(unconditional)
+    }
+    if unconditional:
+        poi_val = poi.getVal()
+        results['poi_bestfit'] = poi_val
+    results['time'] = end-start
+    
     # save results
     if outname is not None:
-        results = {
-            'nll': nll,
-            'poi': poi.GetName(),
-            'constrain': int(constrain),
-            'poi_value': poi_val,
-            'poi_min': poi_min,
-            'poi_max': poi_max,
-            'unconditional': int(unconditional),
-            'time': end-start
-        }
         with open(outname, 'w') as outfile:
             json.dump(results, outfile)
         print('INFO: Saved NLL result to {}'.format(outname))
+        
+    if detailed_output:
+        return results
     return nll
 
 def scan_nll(input_file:str, poi_min:float, poi_max:float, poi_step:float, poi:str="", cache:bool=True,
