@@ -313,7 +313,8 @@ class NuisanceParameterPull(object):
                          repeat(workspace), repeat(model_config), repeat(minimizer_type), 
                          repeat(minimizer_algo), repeat(precision), repeat(eps), repeat(strategy), 
                          repeat(num_cpu), repeat(offset), repeat(optimize), repeat(eigen), 
-                         repeat(fix_cache), repeat(fix_multi), repeat(max_calls), repeat(max_iters), 
+                         repeat(fix_cache), repeat(fix_multi), repeat(max_calls), repeat(max_iters),
+                         repeat(batch_mode), repeat(int_bin_precision),
                          repeat(outdir), repeat(cache), logfile_paths)
             max_workers = parallel if parallel != -1 else common_utils.get_cpu_count()
             common_utils.parallel_run(self._run_pulls, *arguments, max_workers=max_workers)
@@ -323,6 +324,7 @@ class NuisanceParameterPull(object):
                                 binned, workspace, model_config, minimizer_type, 
                                 minimizer_algo, precision, eps, strategy, num_cpu, offset, 
                                 optimize, eigen, fix_cache, fix_multi, max_calls, max_iters,
+                                batch_mode, int_bin_precision,
                                 outdir, cache, logfile_paths[i])
         
     @staticmethod
