@@ -6,17 +6,42 @@ from matplotlib.lines import Line2D
 from matplotlib.ticker import (MultipleLocator, FormatStrFormatter,
                                AutoMinorLocator)
 
-def format_axis_ticks(ax, x_axis=True, y_axis=True, major_length=16, minor_length=8):
+COLOR_PALLETE = ['#000000', '#F2385A', '#4AD9D9', '#FDC536', '#125125', '#E88EED']
+
+
+PLOT_STYLES = {
+    'distributions': {
+        'marker': 'o',
+        'markersize': 1,
+        'linewidth': 1
+    }
+}
+
+def suggest_markersize(nbins:int):
+    if nbins <= 20:
+        return 10
+    elif (nbins > 20) and (nbins <= 200):
+        return nbins/20
+    return 1
+
+def format_axis_ticks(ax, x_axis=True, y_axis=True, major_length=16, minor_length=8,
+                      width=1, labelsize=5):
     if x_axis:
         if ax.get_xaxis().get_scale() != 'log':
             ax.xaxis.set_minor_locator(AutoMinorLocator())
-        ax.tick_params(axis="x", which="major", direction='in', length=major_length)
-        ax.tick_params(axis="x", which="minor", direction='in', length=minor_length)        
+        ax.tick_params(axis="x", which="major", direction='in', length=major_length,
+                       width=width, labelsize=labelsize)
+        ax.tick_params(axis="x", which="minor", direction='in', length=minor_length,
+                       width=width, labelsize=labelsize)
     if y_axis:
         if ax.get_yaxis().get_scale() != 'log':
             ax.yaxis.set_minor_locator(AutoMinorLocator())        
-        ax.tick_params(axis="y", which="major", direction='in', length=major_length)
-        ax.tick_params(axis="y", which="minor", direction='in', length=minor_length) 
+        ax.tick_params(axis="y", which="major", direction='in', length=major_length,
+                       width=width, labelsize=labelsize)
+        ax.tick_params(axis="y", which="minor", direction='in', length=minor_length,
+                       width=width, labelsize=labelsize)
+    for axis in ['top','bottom','left','right']:
+        ax.spines[axis].set_linewidth(width)
 
 def centralize_axis(ax, which='x'):
     if which == 'x':
@@ -97,4 +122,41 @@ def draw_ATLAS_label(axis, x=0.05, y=0.95, fontsize=25, extra='Internal',
                            horizontalalignment='left', verticalalignment='bottom')
     plt.sca(current_axis)
     _, xmax, _, ymax = get_box_dimension(text_atlas)
-    return xmin, xmax, ymin, ymax
\ No newline at end of file
+    return xmin, xmax, ymin, ymax
+
+
+def add_collective_data(ax, data, plot_styles=None):
+    if plot_styles is None:
+        plot_styles = None
+    color_iter = iter(COLOR_PALLETE)
+    for label in data:
+        styles = plot_styles.copy()
+        x = data[label]['x']
+        y = data[label]['y']
+        nbins = len(x)
+        if 'markersize' not in styles:
+            styles['markersize'] = suggest_markersize(nbins)        
+        if 'color' not in styles:
+            styles['color'] = next(color_iter)
+        styles['label'] = label
+        ax.plot(x, y, **styles)
+
+def plot_distributions(data, xlabel='', ylabel='', 
+                       figsize=(14, 8), plot_styles=None, atlas_label=False):
+    plt.clf()
+    base_size = figsize[0]
+    fig = plt.figure(figsize=figsize)
+    ax = plt.gca()
+    styles = PLOT_STYLES['distributions'].copy()
+    if plot_styles is not None:
+        styles.update(plot_styles)
+    add_collective_data(ax, data, styles)
+    limits = ax.get_ylim()
+    ax.set_ylim(limits[0], limits[1]*1.2)
+    ax.set_xlabel(xlabel, fontsize=base_size*1.6,  labelpad=base_size*0.7)
+    ax.set_ylabel(ylabel, fontsize=base_size*1.6,  labelpad=base_size*0.7)
+    format_axis_ticks(ax, major_length=14, minor_length=7, width=1.3, labelsize=base_size*1.4)
+    if atlas_label:
+        draw_ATLAS_label(ax, fontsize=base_size*1.8)
+    ax.legend(fontsize=base_size*1.5)
+    return plt
\ No newline at end of file
